searchNodes=[{"doc":"Protocol for defining behaviour of changesets on discriminator-enabled schemas. In most cases there is no need to alter this logic, however if there is some extra logic needed to be executed everytime some diverged schema changeset is being created it can be injected by overriding this protocol. Could be useful especially for some side effects that doesn't look &quot;good&quot; when put directly inside base changeset function","ref":"EctoDiscriminator.DiscriminatorSchema.html","title":"EctoDiscriminator.DiscriminatorSchema","type":"protocol"},{"doc":"Calls changeset/2 function from base type. Use this inside changeset function of a diverged schema to call base changeset. By default it calls changeset/2 from base schema to apply logic for common fields and then returns changeset for further modifications. It can be placed anywhere in the changeset, but the safest place should be the beginning. cast_base/3 won't contain changes for fields that are overriden by diverged schema. Every module that uses EctoDiscriminator.Schema derives the basic implementation. There is also a helper method to avoid aliasing EctoDiscriminator.DiscriminatorSchema everywhere. Examples def changeset ( struct , params ) do struct |&gt; ... |&gt; cast_base ( params ) |&gt; ... end","ref":"EctoDiscriminator.DiscriminatorSchema.html#cast_base/3","title":"EctoDiscriminator.DiscriminatorSchema.cast_base/3","type":"function"},{"doc":"Calls changeset/2 function from diverged type. In general, this function works the same like calling changeset/2 on diverged schema with cast_base/3 inside. It becomes useful when you want to create changeset for diverged struct, but the data is coming from external source (like user input). This makes it possible to use generic naming like SomeTable in places where there could be multiple diverged types inserted by the same function, to keep code clean. Type is inferred from the discriminator field in params or from passed struct. Every module that uses EctoDiscriminator.Schema derives the basic implementation. There is also a helper method to avoid aliasing EctoDiscriminator.DiscriminatorSchema everywhere. Examples changeset = DiscriminatorSchema . diverged_changeset ( % SomeTable { } , %{ type : SomeTable.Foo } ) changeset . data #=&gt; %SomeTable.Foo{...} You can call the same thing from any module that uses EctoDiscriminator.schema : changeset = SomeTable . diverged_changeset ( % SomeTable { } , %{ type : SomeTable.Foo } ) changeset . data #=&gt; %SomeTable.Foo{...} If type is missing from params it will be taken from the passed struct: changeset = SomeTable . diverged_changeset ( % SomeTable.Foo { } , %{ title : &quot;abc&quot; } ) changeset . data #=&gt; %SomeTable.Foo{...}","ref":"EctoDiscriminator.DiscriminatorSchema.html#diverged_changeset/2","title":"EctoDiscriminator.DiscriminatorSchema.diverged_changeset/2","type":"function"},{"doc":"","ref":"EctoDiscriminator.DiscriminatorSchema.html#t:t/0","title":"EctoDiscriminator.DiscriminatorSchema.t/0","type":"type"},{"doc":"A custom type used by the EctoDiscriminator.Schema to know which field should be used for its logic. EctoDiscriminator.DiscriminatorType field holds name of the diverged module. Field name can be anything. User has to mark exactly one field in the base schema with this type. Diverged schemas will inherit this setting. Forgetting to add this field will result in compilation error. Basic setup: field :type , EctoDiscriminator.DiscriminatorType You can also use it as a primary key: @primary_key { :id , EctoDiscriminator.DiscriminatorType , [ ] } You can configure any options that are supported by Ecto.Schema (except virtual and default from obvious reasons). field :discriminator , EctoDiscriminator.DiscriminatorType , load_in_query : false When it comes to the migrations, the underlying type is just :string . That said, it's completely fine to create migration with string field: add :type , :string It's possible to be more restrictive and provide an enum inside DB. Keep in mind that module name will be stored in form Elixir.Module.Name .","ref":"EctoDiscriminator.DiscriminatorType.html","title":"EctoDiscriminator.DiscriminatorType","type":"module"},{"doc":"Callback implementation for Ecto.Type.cast/1 .","ref":"EctoDiscriminator.DiscriminatorType.html#cast/1","title":"EctoDiscriminator.DiscriminatorType.cast/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.dump/1 .","ref":"EctoDiscriminator.DiscriminatorType.html#dump/1","title":"EctoDiscriminator.DiscriminatorType.dump/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.embed_as/1 .","ref":"EctoDiscriminator.DiscriminatorType.html#embed_as/1","title":"EctoDiscriminator.DiscriminatorType.embed_as/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.equal?/2 .","ref":"EctoDiscriminator.DiscriminatorType.html#equal?/2","title":"EctoDiscriminator.DiscriminatorType.equal?/2","type":"function"},{"doc":"Callback implementation for Ecto.Type.load/1 .","ref":"EctoDiscriminator.DiscriminatorType.html#load/1","title":"EctoDiscriminator.DiscriminatorType.load/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.type/0 .","ref":"EctoDiscriminator.DiscriminatorType.html#type/0","title":"EctoDiscriminator.DiscriminatorType.type/0","type":"function"},{"doc":"Wrapper around Ecto.Schema that enables inheritance of schema fields. It has been built to mimic Ecto.Schema as much as possible. Base schema To make a base schema you have to do two things: Change use Ecto.Schema to use EctoDiscriminator.Schema : defmodule SomeTable do use EctoDiscriminator.Schema Add EctoDiscriminator.DiscriminatorType field to schema (check module documentation for more examples): schema &quot;some_table&quot; do field :type , EctoDiscriminator.DiscriminatorType ... end Base schemas should ideally contain only fields that are common across all diverged schemas (like timestamps). There is no problem with having other fields defined if someone needs it for some functionality though. Any field can be overriden by diverged schema. Diverged changeset Base schemas have predefined function, based on DiscriminatorSchema.diverged_changeset/2 , that allows creating changesets for diverged schemas directly from base (check here for more). Diverged schema To make a diverged schema you have to do two things: Change use Ecto.Schema to use EctoDiscriminator.Schema : defmodule SomeTable.Foo do use EctoDiscriminator.Schema Define schema with name of base schema as a source: schema SomeTable do field ... ... end Diverged schemas can contain any field supported by Ecto.Schema . Casting base fields Diverged schemas have predefined function, based on DiscriminatorSchema.cast_base/3 , that allows running base changesets inside changeset of diverged schema (check here for more). Querying Diverged schemas have some logic injected that allows very simple querying: MyApp.Repo . all ( SomeTable.Foo ) This will generate SQL similar to this: SELECT ... FROM some_table WHERE discriminator = &quot;Elixir.SomeTable.Foo&quot; This functionality should be enough in most cases, however if the injected where condition causes some issues (eg. in some advanced SQL) you can exclude it on the beginning: SomeTable.Foo |&gt; exclude ( :where ) |&gt; MyApp.Repo . all ( ) #=&gt; SELECT ... FROM some_table","ref":"EctoDiscriminator.Schema.html","title":"EctoDiscriminator.Schema","type":"module"},{"doc":"Main building block for inheritance logic. schema/2 wraps Ecto.Schema.schema/2 , adding some features required to support idea of this library. For base schema, source should be the name of a DB table: schema &quot;some_table&quot; do For diverged schema, source must be the name of a base module: schema SomeTable do Additionally, inside schema/2 of the base schema you have to add field which will be acting as a discriminator: field :type , EctoDiscriminator.DiscriminatorType","ref":"EctoDiscriminator.Schema.html#schema/2","title":"EctoDiscriminator.Schema.schema/2","type":"macro"},{"doc":"","ref":"readme.html","title":"EctoDiscriminator","type":"extras"},{"doc":"This small library was built to support table-per-hierarchy inheritance pattern (TPH) popular in Microsoft's Entity Framework. TPH uses a single table to store the data for all types in the hierarchy, and a discriminator column is used to identify which type each row represents. It is similar to Polymorphic Embed with few key differences. Thanks to this library, those entities can be fully separated structs, which brings many simplifications during inserting and querying. You can also add any extra fields that will exist only in one struct (for example virtual ones or relationships). Inheritance, huh?! It may seem not reasonable to introduce concept of inheritance to the Ecto (and Elixir itself), but for some cases I think it's better to have it instead of repeated code. Inheritance is natural for our environment, everything comes from some more general being and shares its capabilities. Without it, you're left just with pattern matching, and it's enough for most cases, but not all of them. Quick example: mug and cup, they look almost the same, but cup can have reference to a saucer. You could preload this reference for both and just remember that for mugs it's always empty, but this generates extra load on the DB (which doesn't know what you know) and forces you to explain for any new person to the project why this will be missing for mug So yea, this library introduces a concept of inheritance to your code.","ref":"readme.html#motivation","title":"EctoDiscriminator - Motivation","type":"extras"},{"doc":"The package can be installed by adding ecto_discriminator to your list of dependencies in mix.exs : def deps do [ { :ecto_discriminator , &quot;~&gt; 0.2.0&quot; } ] end","ref":"readme.html#installation","title":"EctoDiscriminator - Installation","type":"extras"},{"doc":"Schema It has been built to mimic Ecto.Schema as much as possible. That said, the only changes to do in base schema are: Replace Schema module defmodule EctoDiscriminator.SomeTable do - use Ecto.Schema + use EctoDiscriminator.Schema Define discriminator column schema &quot;some_table&quot; do + field :type, EctoDiscriminator.DiscriminatorType You can also mark the primary key as a discriminator + @primary_key {:type, EctoDiscriminator.DiscriminatorType, []} schema &quot;some_table&quot; do Then you can add some diverged schemas defmodule EctoDiscriminator.SomeTable.Foo do use EctoDiscriminator.Schema schema EctoDiscriminator.SomeTable do embeds_one :content , EctoDiscriminator.SomeTable.BarContent end end Library will do the rest. Querying for diverged schema automatically adds filter to SQL. Changeset To reduce repetitive usage of cast with a list of common fields for diverged schemas you can call cast_base(params) to automatically apply changeset from base schema. This function won't be available if base schema doesn't have any changeset/2 function Some may find it useful to insert diverged schema directly from the base (by specifying discriminator value in changeset params). This is doable by calling diverged_changeset function on base schema.","ref":"readme.html#usage","title":"EctoDiscriminator - Usage","type":"extras"},{"doc":"Refer to the documentation of modules for some examples. You can also browse test directory for some example setup.","ref":"readme.html#examples","title":"EctoDiscriminator - Examples","type":"extras"},{"doc":"It's not possible to obtain correct mapping with something like Repo.all(BaseSchema) . You have to execute separate query for each diverged type and then concat results.","ref":"readme.html#known-limitations","title":"EctoDiscriminator - Known limitations","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Bug fix Changed way how DiscriminatorSchema is used. It should be more flexible now. Previously sometimes the changes weren't properly reflected","ref":"changelog.html#v0-2-6","title":"Changelog - v0.2.6","type":"extras"},{"doc":"Bug fix Fixed diverged_changeset/2 default implementation. Previous logic wasn't creating proper diverged struct.","ref":"changelog.html#v0-2-5","title":"Changelog - v0.2.5","type":"extras"},{"doc":"Enhancements Added documentation for the project cast_base/3 now won't return changes for overriden fields due to possible type differences.","ref":"changelog.html#v0-2-4","title":"Changelog - v0.2.4","type":"extras"},{"doc":"Enhancements Major refactor of EctoDiscriminator.Schema","ref":"changelog.html#v0-2-3","title":"Changelog - v0.2.3","type":"extras"}]